<?php

/* MTR Graphs, will create:
 * - target host (loss,avg,worst,best,stddev, warning/crit levels)
 * - combined avgs (avg per hop (inc target) with different colors)
 * - per hop (loss,avg,worst,best,stddev)
 */

setlocale(LC_ALL, "POSIX");

// Make data sources available via names
$RRD = array();
foreach ($NAME as $i => $n)
{
	$RRD[$n]     = "$RRDFILE[$i]:$DS[$i]:MAX";
	$RRD_MIN[$n] = "$RRDFILE[$i]:$DS[$i]:MIN";
	$RRD_AVG[$n] = "$RRDFILE[$i]:$DS[$i]:AVERAGE";
	$DS[$n] = $DS[$i];
	$RRDFILE[$n] = $RRDFILE[$i];
	$WARN[$n] = $WARN[$i];
	$CRIT[$n] = $CRIT[$i];
	$MIN[$n]  = $MIN[$i];
	$MAX[$n]  = $MAX[$i];
	$ACT[$n]  = $ACT[$i];
}

$COLORS['loss']        = '#0000009f';
$COLORS['loss_warn']   = '#995555';
$COLORS['loss_crit']   = '#ff3333';
$COLORS['stddev']      = '#22ffff';
$COLORS['stddev_warn'] = '#22ffcc';
$COLORS['stddev_crit'] = '#22aaaa';
$COLORS['best']        = '#ffcc00';
$COLORS['avg']         = '#996600';
$COLORS['avg_warn']    = '#996699';
$COLORS['avg_crit']    = '#9966ff';
$COLORS['worst']       = '#ff000001#ff0000ff';
$COLORS['target']      = '#ff9933';

$RAINBOW = array('#f22', '#f2f', '#22f', '#2ff', '#2f2', '#ff2','#922', '#929', '#229', '#299', '#292', '#992');

$LABELSIZE = 18;

# RRDtool Options
$targethost = substr($servicedesc,7);

$opt[1] = "--vertical-label ms -l 0 --title '$hostname: MTR to $targethost' --tabwidth 5 ";
$ds_name[1] = "$targethost";

# First graph the destination
# $def[1] = "DEF:loss=${RRD_AVG['loss']} ";
$def[1]  = rrd::def("loss"    , $RRDFILE['loss']    , $DS['loss']    , "AVERAGE");
# Scale loss to max(worst)
$def[1] .= rrd::def("avg"     , $RRDFILE["avg"]     , $DS["avg"]     , "AVERAGE");
$def[1] .= rrd::def("best"    , $RRDFILE["best"]    , $DS["best"]    , "MIN");
$def[1] .= rrd::def("worst"   , $RRDFILE["worst"]   , $DS["worst"]   , "MAX");
$def[1] .= rrd::def("stddev"  , $RRDFILE["stddev"]  , $DS["stddev"]  , "AVERAGE");
$def[1] .= rrd::def("hopcount", $RRDFILE["hopcount"], $DS["hopcount"], "AVERAGE");
$def[1] .= "VDEF:worstmax=worst,MAXIMUM ";
$def[1] .= "CDEF:smoke=best,stddev,+ ";
$wmaxrpn = "loss,100,/,worstmax,0,GT,worstmax,100,IF,* "; # Scale loss to highest numbers (which should be 'worst'), If worstmax is zero (which happens if all packets are dropped) we'll use 100 instead, so the graph will show full loss.
$def[1] .= "CDEF:sloss=$wmaxrpn ";

$def[1] .= "GPRINT:hopcount:LAST:\"  ".rrd::cut("Total hopcount",$LABELSIZE)."\\t  cur\:%6.0lf  \\t\" ";
$def[1] .= "GPRINT:hopcount:MIN:\"  min\:%6.0lf\\t\" ";
$def[1] .= "GPRINT:hopcount:AVERAGE:\"  avg\:%6.0lf\\t\" ";
$def[1] .= "GPRINT:hopcount:MAX:\"  max\:%6.0lf\\n\" ";

$def[1] .= "AREA:worst".$COLORS['worst'].":\"".rrd::cut("worst roundtrip", $LABELSIZE)."\\t\":gradheight=0 ";
$def[1] .= "GPRINT:worst:LAST:\"cur\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:worst:MIN:\"min\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:worst:AVERAGE:\"avg\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:worst:MAX:\"max\: %6.2lfms\\n\" ";

# $def[1] .= "AREA:smoke#ff0000#ffcc00:gradheight=50 ";
$def[1] .= "AREA:smoke#ff99008f#ffff00:gradheight=0 ";
#$def[1] .= rrd::gradient('smoke', 'ffcc00', 'ff0000', null, 5); # Best to worst colored , no label
# $def[1] .= rrd::gradient('smoke', '000000ff', 'ffffff00', null, 10); # Best to worst colored , no label
# $def[1] .= rrd::gradient('smoke', 'ffcc00ff', '0000ffff', null, 1000);
#$def[1] .= "LINE1:smoke#0000FF ";

$def[1] .= "AREA:avg".$COLORS['best']." ";
$def[1] .= "LINE3:avg".$COLORS['avg'].":\"".rrd::cut("  avg roundtrip", $LABELSIZE)."\\t\" ";
$def[1] .= "GPRINT:avg:LAST:\"cur\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:avg:MIN:\"min\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:avg:AVERAGE:\"avg\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:avg:MAX:\"max\: %6.2lfms\\n\" ";

$def[1] .= "AREA:best#FFFFFF ";
$def[1] .= "LINE2:best".$COLORS['best'].":\"".rrd::cut(" best roundtrip", $LABELSIZE)."\\t\" ";
$def[1] .= "GPRINT:best:LAST:\"cur\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:best:MIN:\"min\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:best:AVERAGE:\"avg\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:best:MAX:\"max\: %6.2lfms\\n\" ";

$def[1] .= "LINE1:stddev".$COLORS['stddev'].":\"".rrd::cut("stddev roundtrip", $LABELSIZE)."\\t\" ";
$def[1] .= "GPRINT:stddev:LAST:\"cur\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:stddev:MIN:\"min\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:stddev:AVERAGE:\"avg\: %6.2lfms\\t\" ";
$def[1] .= "GPRINT:stddev:MAX:\"max\: %6.2lfms\\n\" ";

$def[1] .= "AREA:sloss".$COLORS['loss'].":\"".rrd::cut("packet loss %", $LABELSIZE)."\\t\" ";
$def[1] .= "GPRINT:loss:LAST:\"cur\: %6.2lf%%\\t\" ";
$def[1] .= "GPRINT:loss:MIN:\"min\: %6.2lf%% \\t\" ";
$def[1] .= "GPRINT:loss:AVERAGE:\"avg\: %6.2lf%%\\t\" ";
$def[1] .= "GPRINT:loss:MAX:\"max\: %6.2lf%% \\n\" ";

# Warns / crits at end
if($WARN["loss"] != "" && $WARN["loss"] > 0)
{
	$def[1] .= "CDEF:losswarnc=loss,0,*,".$WARN["loss"].",100,/,worstmax,0,GT,worstmax,100,IF,*,+ ";
	$def[1] .= "VDEF:losswarn=losswarnc,MAXIMUM ";
	$def[1] .= rrd::hrule("losswarn", $COLORS["loss_warn"], rrd::cut("Packet loss warn", $LABELSIZE)."\:\\t".$WARN["loss"]."%\\t");
}
if($CRIT["loss"] != "" && $CRIT["loss"] > 0)
{
	$def[1] .= "CDEF:losscritc=loss,0,*,".$CRIT["loss"].",100,/,worstmax,0,GT,worstmax,100,IF,*,+ ";
	$def[1] .= "VDEF:losscrit=losscritc,MAXIMUM ";
	$def[1] .= rrd::hrule("losscrit", $COLORS["loss_crit"], rrd::cut("Packet loss crit", $LABELSIZE)."\:\\t".$CRIT["loss"]."%\\n");
}
if($WARN["avg"] != "" && $WARN["avg"] > 0) $def[1] .= rrd::hrule($WARN["avg"], $COLORS["avg_warn"], rrd::cut("Average warning", $LABELSIZE)."\:\\t".$WARN["avg"]."ms\\t");
if($CRIT["avg"] != "" && $CRIT["avg"] > 0) $def[1] .= rrd::hrule($CRIT["avg"], $COLORS["avg_crit"], rrd::cut("Average crit", $LABELSIZE)."\:\\t".$CRIT["avg"]."ms\\n");
if($WARN["stddev"] != "" && $WARN["stddev"] > 0) $def[1] .= rrd::hrule($WARN["stddev"], $COLORS["stddev_warn"], rrd::cut("Stddev warning", $LABELSIZE)."\:\\t".$WARN["stddev"]."ms\\t");
if($CRIT["stddev"] != "" && $CRIT["stddev"] > 0) $def[1] .= rrd::hrule($CRIT["stddev"], $COLORS["stddev_crit"], rrd::cut("Stddev critical", $LABELSIZE)."\:\\t".$CRIT["stddev"]."ms\\n");

# Combined hops etc
$opt[2] = "--vertical-label 'ms' -l 0 --title '$hostname: MTR to $targethost, roundtrip avg with all nodes' ";
$ds_name[2] = "Hop overview";
$def[2] = rrd::def("avg", $RRDFILE["avg"], $DS["avg"], "AVERAGE");

for ($hop = 1; isset($RRD_AVG['hop_'.$hop.'_loss']); $hop++)
{
	$def[2] .= rrd::def("avg".$hop   , $RRDFILE["hop_${hop}_avg"]   , $DS["hop_${hop}_avg"]   , "AVERAGE");
	$color = array_shift($RAINBOW); array_push($RAINBOW, $color);
	$hlabel = rrd::cut("Hop $hop", $LABELSIZE);
	$def[2] .= "LINE1:avg$hop".$color.":\"$hlabel\\t\" ";
	$def[2] .= "GPRINT:avg$hop:MIN:\"min\: %6.2lf ms\\t\" ";
	$def[2] .= "GPRINT:avg$hop:AVERAGE:\"avg\: %6.2lf ms\\t\" ";
	$def[2] .= "GPRINT:avg$hop:MAX:\"max\: %6.2lf ms\\n\" ";
}
$tlabel = rrd::cut($targethost, $LABELSIZE);
$def[2] .= "LINE2:avg".$COLORS['target'].":\"$tlabel\\t\" ";
$def[2] .= "GPRINT:avg:MIN:\"min\: %6.2lf ms\\t\" ";
$def[2] .= "GPRINT:avg:AVERAGE:\"avg\: %6.2lf ms\\t\" ";
$def[2] .= "GPRINT:avg:MAX:\"max\: %6.2lf ms\\n\" ";

# Second graph and further (which are optional) are the hops in between the host and destination.
$start = 2;
for ($hop = 1; isset($RRD_AVG['hop_'.$hop.'_loss']); $hop++)
{
	$opt[$hop + $start] = "--vertical-label 'ms' -l 0 --title '$hostname: MTR to $targethost, Hop $hop' ";
	$ds_name[$hop + $start] = "Hop $hop";
	$def[$hop + $start]  = rrd::def("loss"  , $RRDFILE["hop_${hop}_loss"]  , $DS["hop_${hop}_loss"]  , "AVERAGE");
	$def[$hop + $start] .= rrd::def("avg"   , $RRDFILE["hop_${hop}_avg"]   , $DS["hop_${hop}_avg"]   , "AVERAGE");
	$def[$hop + $start] .= rrd::def("best"  , $RRDFILE["hop_${hop}_best"]  , $DS["hop_${hop}_best"]  , "AVERAGE");
	$def[$hop + $start] .= rrd::def("worst" , $RRDFILE["hop_${hop}_worst"] , $DS["hop_${hop}_worst"] , "AVERAGE");
	$def[$hop + $start] .= rrd::def("stddev", $RRDFILE["hop_${hop}_stddev"], $DS["hop_${hop}_stddev"], "AVERAGE");
	$def[$hop + $start] .= "VDEF:worstmax=worst,MAXIMUM ";
	$def[$hop + $start] .= "CDEF:smoke=best,stddev,+ ";
	$def[$hop + $start] .= "CDEF:sloss=$wmaxrpn ";

	$def[$hop + $start] .= "AREA:worst".$COLORS['worst'].":\"".rrd::cut(" worst roundtrip", $LABELSIZE)."\\t\" ";
	$def[$hop + $start] .= "GPRINT:worst:LAST:\"cur\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:worst:MIN:\"min\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:worst:AVERAGE:\"avg\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:worst:MAX:\"max\: %6.2lfms\\n\" ";

	$def[$hop + $start] .= "AREA:avg".$COLORS['best']." ";
	$def[$hop + $start] .= "LINE3:avg".$COLORS['avg'].":\"".rrd::cut("   avg roundtrip", $LABELSIZE)."\\t\" ";
	$def[$hop + $start] .= "GPRINT:avg:LAST:\"cur\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:avg:MIN:\"min\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:avg:AVERAGE:\"avg\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:avg:MAX:\"max\: %6.2lfms\\n\" ";

	$def[$hop + $start] .= "AREA:best#FFFFFF ";
	$def[$hop + $start] .= "LINE2:best".$COLORS['best'].":\"".rrd::cut("  best roundtrip", $LABELSIZE)."\\t\" ";
	$def[$hop + $start] .= "GPRINT:best:LAST:\"cur\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:best:MIN:\"min\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:best:AVERAGE:\"avg\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:best:MAX:\"max\: %6.2lfms\\n\" ";

	$def[$hop + $start] .= "LINE1:stddev".$COLORS['stddev'].":\"".rrd::cut("stddev roundtrip", $LABELSIZE)."\\t\" ";
	$def[$hop + $start] .= "GPRINT:stddev:LAST:\"cur\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:stddev:MIN:\"min\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:stddev:AVERAGE:\"avg\: %6.2lfms\\t\" ";
	$def[$hop + $start] .= "GPRINT:stddev:MAX:\"max\: %6.2lfms\\n\" ";

	$def[$hop + $start] .= "AREA:sloss".$COLORS["loss"].":\"".rrd::cut("packet loss %", $LABELSIZE)."\\t\" ";
	$def[$hop + $start] .= "GPRINT:loss:LAST:\"cur\: %6.2lf%%\\t\" ";
	$def[$hop + $start] .= "GPRINT:loss:MIN:\"min\: %6.2lf%%\\t\" ";
	$def[$hop + $start] .= "GPRINT:loss:AVERAGE:\"avg\: %6.2lf%%\\t\" ";
	$def[$hop + $start] .= "GPRINT:loss:MAX:\"max\: %6.2lf%%\\n\" ";

}

?>
